;CODE FOR DMX C4 EMULATOR BOARD
;TO TEST DMX 973B GRAPHC MODULE
;REMOVE IC1, USE RA0 FOR DISPLAY RESET   
;PIC:16C57
;CLOCK:4MHz (1us instruction cycle)
;WDT OFF
;USE XT OPTION
                                                        
;FILE NAME:MAIN.ASM                 
                                
        include "picreg.h"                                  
                                                            
;*******************************************************
;All of the port defines
        #define RS      port_a,1        ;LCD control lines
        #define RW      port_a,2                              
        #define E       port_a,3

        #define RTS     port_c,0        ;RS232 lines
        #define CTS     port_c,1
        #define Txd     port_c,2
        #define Rxd     port_c,3

        #define SCL     port_c,5        ;EEPROM lines
        #define SDA     port_c,4

        #define data    port_b          ;data bus

        #define RESET	port_a,0        ;Display reset 

;*******************************************************
;RAM defines

count   equ     0x08                    ;For delay routines
count2  equ     0x09                    ;For delay routines
temp    equ     0x0a                    ;temp storage during sub routines
addr    equ     0x0f                    ;stores position of cursor 
page973 equ     0x0e                    ;current page of dmx973

ddata	equ	0x0b			;temp storage for display data
temp2	equ	0x0d


;********************************************************

        org     0x00

; Configure output pins as required
setup   movlw   0x9a            ;set port c for i/o
        tris    port_c
        clrf    port_c

        movlw   0x00            ;set port b  for input
        tris    port_b

        movlw   0x00            ;set port for output
        tris    port_a

        bsf     Txd             ;set serial output to high
        bsf     RESET           ;ensure reset is high 
        bsf     RTS             ;set rts high

                        	;set start conditions
        bsf     RS              ;select display data
        bcf    	RW              ;select read
        bcf     E              	;disable

        goto    set973          ;initialise display
  
forever 
        goto forever


;*******************************************************
;Sets up the dmx973 for normal operation

set973  
;       Apply reset pulse
        bcf RESET
        bsf RESET

sete2	movlw   0xe2            ;Reset command
        call    inst973

        movlw	0xa3            ;a3
        call	inst973		;change Bias to 1/5

        movlw   0xa0            ;ADC right is a0
        call    inst973

        movlw	0x9f            ;Max out electronic volume register
        call    inst973

       	movlw   0x2f            ;turn power boost circuit on   2f
        call    inst973 

        movlw   0xaf            ;Display On
        call    inst973

        movlw   0xb0            ;Display page = 0
        movwf   page973
        call    inst973

	movlw   0x11            ;Set high nibble column = 0
        call    inst973

 	movlw   0x00            ;Set low nibble column = 0
        call    inst973

        movlw   0x40            ;Set start line 40   for 973=c0  MSC modules=d0
        call    inst973

        goto    txt973         ;blank display  *get out here*

;;*******************************************************

;sends an instruction to the dmx973

inst973 movwf   temp           ;temporary storage of data

;        bcf     STATUS,PA0    ;page 0
;        bcf     STATUS,PA1
;        call    busy           ;check busy flag

        movlw   0x00           ;set data port for o/p
        tris    data
        
        movfw   temp
        movwf   data           ;put data on bus
        
	bcf     RS             ;select instruction data
        bcf     RW             ;select write

			       ;clock data in
itogE   bsf     E                 
        bcf	E              ;clear enable
   	       
iendtog movlw   0xff           ;put data port back to i/p
        tris    data

;        bsf     STATUS,PA0
;        bsf     STATUS,PA1 

        return




;*******************************************************
;Checks lcd busy flag. Won't return until busy is clear

busy    movlw   0xff            ;set port b for input
        tris    port_b

        bcf     RS              ;select instruction data
        bsf     RW              ;select read

btogE   bsf     E              ;enable chip 1  

        btfsc   data,7          ;check busy bit
        goto    busy
        
	movfw   data
        movwf   addr            ;store address of cursor

        bcf     E               ;disable display IC

        movlw   0x00            ;set port for output
        tris    port_b
        return

;Display message on the 973
txt973  clrf    count
        clrf    count2

        movlw   0xb0            ;Display page = b0   973 b8
        movwf   page973
        call    inst973
		
	movlw   0x00            ;Set lower nibble column = 0
        call    inst973

        movlw   0x11		;Set higher nibble column = 0
        call    inst973


txta973 bcf     STATUS,PA0
        bcf     STATUS,PA1
        call    busy            ;check for busy flag 

        ;get data from look up table
        movfw	count
        bsf     STATUS,PA1      ;set page 2
	call	list_t

        bcf	STATUS,PA1
        bcf     RW
        bsf     RS
        movwf   data
        bsf     E               ;clock data in
        bcf     E               ;by enabling breifly
        incf    count ,1
        movfw   count
        xorlw   0x64            ;3e,64
        btfss   STATUS,Z_BIT
        goto    txta973
        
        incf    page973,1               ;increment the page by 1
        movfw   page973 
        call    inst973                 ;send instruction

        movlw   0x00                    ;reset low column address
        call    inst973                 ;ic1

        movlw   0x11                    ;reset high column address
        call    inst973                 ;ic1

        clrf    count                   ;reset address counter

        incf    count2,1
        movfw   count2
        xorlw   0x04
        btfss   STATUS,Z_BIT
        goto    txta973                  ;do next loop

        movlw   0xff
        tris    data

;        btfsc   port_c,4
        goto    forever


;*******************************************************

;*******************************************************
;Start page 2 of memory
        org     0x400

list_t  addwf   pc,1
        retlw   0x00
        retlw   0x3e
        retlw   0x40
        retlw   0x5f
        retlw   0x50
        retlw   0x57
        retlw   0x16
        retlw   0x00
        retlw   0x00            
	retlw   0x7f            ;Show available display types
        retlw   0x40            ;L
        retlw   0x40
        retlw   0x40
        retlw   0x40
        retlw   0x00            ;space
	retlw   0x7e            ;A
        retlw   0x11
        retlw   0x11
        retlw   0x11            
        retlw   0x7e
        retlw   0x00		;space
        retlw   0x46            ;S
        retlw   0x49
        retlw   0x49
        retlw   0x49
        retlw   0x31
        retlw   0x00            ;space
        retlw   0x3e            ;C
        retlw   0x41            
        retlw   0x41
        retlw   0x41
        retlw   0x22
        retlw   0x00            ;space
        retlw   0x7e            ;A
        retlw   0x11
        retlw   0x11
        retlw   0x11
        retlw   0x7e
        retlw   0x00            
        retlw   0x7f            ;R
        retlw   0x09
        retlw   0x19
        retlw   0x29
        retlw	0x46
        retlw	0x00            ;space
        retlw	0x00       
        retlw	0x00
        retlw	0x00
        retlw	0x00            
        retlw	0x00
        retlw	0x7f            ;D
        retlw	0x41
        retlw	0x41
        retlw	0x22
        retlw	0x1c
        retlw	0x00            ;space
	retlw   0x7f            ;M
        retlw   0x02            
        retlw   0x0c
        retlw   0x02
        retlw   0x7f        
        retlw	0x00            ;space
        retlw	0x63            ;X
        retlw	0x14
        retlw	0x08
        retlw	0x14
        retlw	0x63
        retlw   0x00            ;space
        retlw   0x06            ;9
        retlw   0x49            
        retlw   0x49
        retlw   0x29
        retlw   0x1e        
        retlw   0x00            ;space
        retlw   0x01            ;7
        retlw   0x71            
        retlw   0x09
        retlw   0x05
        retlw   0x03        
        retlw   0x00            ;space
        retlw   0x21            ;3
        retlw   0x41            
        retlw   0x45
        retlw   0x4b
        retlw   0x31        
        retlw   0x00            ;space
        retlw   0x7f            ;B
        retlw   0x49            
        retlw   0x49
        retlw   0x49
        retlw   0x36        
        retlw	0x00            ;space
        retlw   0x12            ;:-)
        retlw   0x20            
        retlw   0x2c
        retlw   0x20
        retlw   0x12   
        retlw	0x00
        retlw	0x55            
        retlw	0xAA
        retlw	0x00
        retlw	0x01
        retlw	0x02
        retlw	0x03
        retlw	0x04
        retlw	0x05
        retlw	0x06
        retlw	0x07
        retlw	0x08
        retlw	0x09
        retlw	0x0A
        retlw	0x0b
        retlw	0x0c
        retlw	0x0d
        retlw	0x0e
        retlw	0x0f
        retlw	0x00
        retlw	0x0f

;******************************************************
        org     PIC57
        goto    setup
        end
